import React, { Component, PropTypes } from 'react';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import * as u from '../utils';
import Slider from './Slider';
import Map from './Map';

class ColorPicker extends Component {
  constructor (props) {
    super(props);
    this.state = {
      color: u.toHSV(this.props.color)
    }
    this.shouldComponentUpdate = PureRenderMixin.shouldComponentUpdate.bind(this);
    this.handleSaturationValueChange = this.handleSaturationValueChange.bind(this);
    this.handleHueChange = this.handleHueChange.bind(this);
    this.handleAlphaChange = this.handleAlphaChange.bind(this);
  }

  handleHueChange (hue) {
    const [, s, v, a] = this.state.color;
    this.update([hue, s, v, a]);
  }

  handleSaturationValueChange(saturation, value) {
    const [h, , , a] = this.state.color;
    this.update([h, saturation, value, a]);
  }

  handleAlphaChange (alpha) {
    const [h, s, v] = this.state.color;
    this.update([h, s, v, alpha]);
  }

  getAlpha () {
    return this.state.color[3] === undefined ? 1 : this.state.color[3];
  }

  getBackgroundHue () {
    return u.toRgbString([this.state.color[0], 100, 100]);
  }

  getBackgroundGradient() {
    const [h, s, v] = this.state.color;
    const opaque = u.toRgbString([h, s, v, 1]);

    return `linear-gradient(to right, rgba(0,0,0,0) 0%, ${opaque} 100%)`;
  }

  update (color) {
    this.setState({ color });
    this.props.onChange(u.toRgbString(color));
  }

  applyOpacitySliderStyles () {

  }

  render () {
    const [ hue, saturation, value ] = this.state.color;
    const {
      colorPicker,
      opacitySlider,
      opacitySlider__slider } = ColorPicker.defaultStyles;
    return (
      <div className="ColorPicker" style={colorPicker}>
        <Slider
          vertical={true}
          value={hue}
          type="hue"
          max={360}
          onChange={this.handleHueChange} />

        {this.props.opacitySlider && (
          <Slider
            className="opacity-slider"
            type="opacity"
            bottom={true}
            value={this.getAlpha()}
            max={1}
            background={this.getBackgroundGradient()}
            onChange={this.handleAlphaChange}
          />
        )}

        <Map
          x={saturation}
          y={value}
          max={100}
          pointerColor={!u.isDark(this.state.color) ? "dark" : "light"}
          backgroundColor={this.getBackgroundHue()}
          onChange={this.handleSaturationValueChange}
        />
      </div>
    );
  }
}

ColorPicker.propTypes = {
  color: PropTypes.string.isRequired,
  onChange: PropTypes.func.isRequired
};

ColorPicker.defaultProps = {
  color: "rgba(0,0,0,1)"
}

ColorPicker.defaultStyles = {
  colorPicker: {
    position: 'absolute',
    top: 0,
    right: 0,
    bottom: 0,
    left: 0,
  },
};

export default ColorPicker;
